@@ -16,6 +16,7 @@
 import copy
 import mock
 
+from keystoneclient.openstack.common.apiclient import exceptions as ksc_exc
 from openstackclient.identity.v2_0 import user
 from openstackclient.tests import fakes
 from openstackclient.tests.identity.v2_0 import fakes as identity_fakes
@@ -83,7 +84,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -91,7 +92,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_password(self):
         arglist = [
@@ -122,7 +123,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -130,7 +131,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_password_prompt(self):
         arglist = [
@@ -163,7 +164,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -171,7 +172,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_email(self):
         arglist = [
@@ -201,7 +202,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -209,7 +210,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_project(self):
         # Return the new project
@@ -254,7 +255,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -262,7 +263,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.PROJECT_2['id'],
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_enable(self):
         arglist = [
@@ -293,7 +294,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -301,7 +302,7 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
 
     def test_user_create_disable(self):
         arglist = [
@@ -332,7 +333,7 @@ class TestUserCreate(TestUser):
         )
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -340,7 +341,85 @@ class TestUserCreate(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
+
+    def test_user_create_or_show_exists(self):
+        def _raise_conflict(*args, **kwargs):
+            raise ksc_exc.Conflict(None)
+
+        # need to make this throw an exception...
+        self.users_mock.create.side_effect = _raise_conflict
+
+        self.users_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.USER),
+            loaded=True,
+        )
+
+        arglist = [
+            '--or-show',
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('name', identity_fakes.user_name),
+            ('or_show', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # UserManager.create(name, password, email, tenant_id=, enabled=)
+        self.users_mock.get.assert_called_with(identity_fakes.user_name)
+
+        collist = ('email', 'enabled', 'id', 'name', 'project_id')
+        self.assertEqual(collist, columns)
+        datalist = (
+            identity_fakes.user_email,
+            True,
+            identity_fakes.user_id,
+            identity_fakes.user_name,
+            identity_fakes.project_id,
+        )
+        self.assertEqual(datalist, data)
+
+    def test_user_create_or_show_not_exists(self):
+        arglist = [
+            '--or-show',
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('name', identity_fakes.user_name),
+            ('or_show', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'enabled': True,
+            'tenant_id': None,
+        }
+        # UserManager.create(name, password, email, tenant_id=, enabled=)
+        self.users_mock.create.assert_called_with(
+            identity_fakes.user_name,
+            None,
+            None,
+            **kwargs
+        )
+
+        collist = ('email', 'enabled', 'id', 'name', 'project_id')
+        self.assertEqual(collist, columns)
+        datalist = (
+            identity_fakes.user_email,
+            True,
+            identity_fakes.user_id,
+            identity_fakes.user_name,
+            identity_fakes.project_id,
+        )
+        self.assertEqual(datalist, data)
 
 
 class TestUserDelete(TestUser):
@@ -381,6 +460,11 @@ class TestUserList(TestUser):
     def setUp(self):
         super(TestUserList, self).setUp()
 
+        self.projects_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.PROJECT_2),
+            loaded=True,
+        )
         self.projects_mock.list.return_value = [
             fakes.FakeResource(
                 None,
@@ -408,15 +492,15 @@ class TestUserList(TestUser):
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=None)
 
         collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.user_id,
             identity_fakes.user_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
     def test_user_list_project(self):
         arglist = [
@@ -426,19 +510,20 @@ class TestUserList(TestUser):
             ('project', identity_fakes.project_id),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        project_id = identity_fakes.PROJECT_2['id']
 
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=project_id)
 
         collist = ('ID', 'Name')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.user_id,
             identity_fakes.user_name,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
     def test_user_list_long(self):
         arglist = [
@@ -452,10 +537,10 @@ class TestUserList(TestUser):
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=None)
 
         collist = ('ID', 'Name', 'Project', 'Email', 'Enabled')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = ((
             identity_fakes.user_id,
             identity_fakes.user_name,
@@ -463,7 +548,7 @@ class TestUserList(TestUser):
             identity_fakes.user_email,
             True,
         ), )
-        self.assertEqual(tuple(data), datalist)
+        self.assertEqual(datalist, tuple(data))
 
 
 class TestUserSet(TestUser):
@@ -731,7 +816,7 @@ class TestUserShow(TestUser):
         self.users_mock.get.assert_called_with(identity_fakes.user_id)
 
         collist = ('email', 'enabled', 'id', 'name', 'project_id')
-        self.assertEqual(columns, collist)
+        self.assertEqual(collist, columns)
         datalist = (
             identity_fakes.user_email,
             True,
@@ -739,4 +824,4 @@ class TestUserShow(TestUser):
             identity_fakes.user_name,
             identity_fakes.project_id,
         )
-        self.assertEqual(data, datalist)
+        self.assertEqual(datalist, data)
